@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,15 +25,17 @@
 
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_socket.h>
+#include <fluent-bit/flb_network.h>
 #include <fluent-bit/flb_config.h>
 
 #ifdef FLB_HAVE_TLS
 #include <mbedtls/net.h>
 #endif
+
 /*
  * Upstream creation FLAGS set by Fluent Bit sub-components
  * ========================================================
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *
  * --- flb_io.h ---
  *   #define  FLB_IO_TCP      1
@@ -53,9 +55,8 @@ struct flb_upstream {
 
     int n_connections;
 
-
-    /* Keepalive */
-    int ka_timeout;    /* maximum number of seconds that a connection can exists */
+    /* Networking setup for timeouts and network interfaces */
+    struct flb_net_setup net;
 
     /*
      * If an upstream context has been created in HA mode, this flag is
@@ -82,6 +83,8 @@ struct flb_upstream {
     /* context with mbedTLS data to handle certificates and keys */
     struct flb_tls *tls;
 #endif
+
+    struct mk_list _head;
 };
 
 /* Upstream TCP connection */
@@ -92,13 +95,33 @@ struct flb_upstream_conn {
     /* Socker */
     flb_sockfd_t fd;
 
+    /*
+     * Recycle: if the connection is keepalive, this flag is always on, but if
+     * the caller wants to drop the connection once is released, it can set
+     * recycle to false.
+     */
+    int recycle;
+
     /* Keepalive */
     int ka_count;        /* how many times this connection has been used */
 
+    /*
+     * Custom 'error' for the connection file descriptor. Commonly used to
+     * specify a reason for an exception that was generated locally: consider
+     * a connect timeout, we shutdown(2) the connection but in reallity we
+     * might want to express an 'ETIMEDOUT'
+     */
+    int net_error;
+
     /* Timestamps */
+    time_t ts_assigned;
     time_t ts_created;
     time_t ts_available;  /* sets the 'start' available time */
 
+    /* Connect */
+    time_t ts_connect_start;
+    time_t ts_connect_timeout;
+
     /* Upstream parent */
     struct flb_upstream *u;
 
@@ -125,7 +148,12 @@ struct flb_upstream *flb_upstream_create_url(struct flb_config *config,
 
 int flb_upstream_destroy(struct flb_upstream *u);
 
+int flb_upstream_conn_recycle(struct flb_upstream_conn *conn, int val);
 struct flb_upstream_conn *flb_upstream_conn_get(struct flb_upstream *u);
 int flb_upstream_conn_release(struct flb_upstream_conn *u_conn);
+int flb_upstream_conn_timeouts(struct flb_config *ctx);
+int flb_upstream_set_property(struct flb_config *config,
+                              struct flb_net_setup *net, char *k, char *v);
+struct mk_list *flb_upstream_get_config_map(struct flb_config *config);
 
 #endif
