@@ -72,7 +72,7 @@ struct cio_ctx {
      * memory. For short, it restrict the open number of files and
      * the amount of memory mapped.
      */
-    int max_chunks_up;
+    size_t max_chunks_up;
 
     /* streams */
     struct mk_list streams;
@@ -85,6 +85,7 @@ struct cio_ctx *cio_create(const char *root_path,
                            void (*log_cb), int log_level, int flags);
 void cio_destroy(struct cio_ctx *ctx);
 int cio_load(struct cio_ctx *ctx);
+int cio_qsort(struct cio_ctx *ctx, int (*compar)(const void *, const void *));
 
 void cio_set_log_callback(struct cio_ctx *ctx, void (*log_cb));
 int cio_set_log_level(struct cio_ctx *ctx, int level);
