@@ -122,6 +122,66 @@ int cio_load(struct cio_ctx *ctx)
     return 0;
 }
 
+static int qsort_stream(struct cio_stream *stream,
+                        int (*compar)(const void *, const void *))
+{
+    int i = 0;
+    int items;
+    struct mk_list *tmp;
+    struct mk_list *head;
+    struct cio_chunk **arr;
+    struct cio_chunk *chunk;
+
+    items = mk_list_size(&stream->chunks);
+    if (items == 0) {
+        return 0;
+    }
+
+    arr = malloc(sizeof(struct cio_chunk *) * items);
+    if (!arr) {
+        perror("malloc");
+        return -1;
+    }
+
+    /* map chunks to the array and and unlink them */
+    mk_list_foreach_safe(head, tmp, &stream->chunks) {
+        chunk = mk_list_entry(head, struct cio_chunk, _head);
+        arr[i++] = chunk;
+        mk_list_del(&chunk->_head);
+    }
+
+    /* sort the chunks, just trust in 'compar' external function  */
+    qsort(arr, items, sizeof(struct cio_chunk *), compar);
+
+    /* link the chunks in the proper order back to the list head */
+    for (i = 0; i < items; i++) {
+        chunk = arr[i];
+        mk_list_add(&chunk->_head, &stream->chunks);
+    }
+
+    free(arr);
+    return 0;
+}
+
+/*
+ * Sort chunks using the 'compar' callback function. This is pretty much a
+ * wrapper over qsort(3). The sort is done inside every stream content.
+ *
+ * Use this function after cio_load() only.
+ */
+int cio_qsort(struct cio_ctx *ctx, int (*compar)(const void *, const void *))
+{
+    struct mk_list *head;
+    struct cio_stream *stream;
+
+    mk_list_foreach(head, &ctx->streams) {
+        stream = mk_list_entry(head, struct cio_stream, _head);
+        qsort_stream(stream, compar);
+    }
+
+    return 0;
+}
+
 void cio_destroy(struct cio_ctx *ctx)
 {
     if (!ctx) {
