@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,9 +26,11 @@
 #include <fluent-bit/flb_mem.h>
 #include <fluent-bit/flb_str.h>
 #include <fluent-bit/flb_filter.h>
+#include <fluent-bit/flb_filter_plugin.h>
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_regex.h>
+#include <fluent-bit/flb_record_accessor.h>
 #include <msgpack.h>
 
 #include "grep.h"
@@ -41,8 +43,9 @@ static void delete_rules(struct grep_ctx *ctx)
 
     mk_list_foreach_safe(head, tmp, &ctx->rules) {
         rule = mk_list_entry(head, struct grep_rule, _head);
-        flb_free(rule->field);
+        flb_sds_destroy(rule->field);
         flb_free(rule->regex_pattern);
+        flb_ra_destroy(rule->ra);
         flb_regex_destroy(rule->regex);
         mk_list_del(&rule->_head);
         flb_free(rule);
@@ -51,6 +54,7 @@ static void delete_rules(struct grep_ctx *ctx)
 
 static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
 {
+    flb_sds_t tmp;
     struct mk_list *head;
     struct mk_list *split;
     struct flb_split_entry *sentry;
@@ -76,7 +80,7 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
             rule->type = GREP_EXCLUDE;
         }
         else {
-            flb_error("[filter_grep] unknown rule type '%s'", kv->key);
+            flb_plg_error(ctx->ins, "unknown rule type '%s'", kv->key);
             delete_rules(ctx);
             flb_free(rule);
             return -1;
@@ -85,7 +89,8 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
         /* As a value we expect a pair of field name and a regular expression */
         split = flb_utils_split(kv->val, ' ', 1);
         if (mk_list_size(split) != 2) {
-            flb_error("[filter_grep] invalid regex, expected field and regular expression");
+            flb_plg_error(ctx->ins,
+                          "invalid regex, expected field and regular expression");
             delete_rules(ctx);
             flb_free(rule);
             flb_utils_split_free(split);
@@ -94,8 +99,17 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
 
         /* Get first value (field) */
         sentry = mk_list_entry_first(split, struct flb_split_entry, _head);
-        rule->field = flb_strndup(sentry->value, sentry->len);
-        rule->field_len = sentry->len;
+        if (*sentry->value == '$') {
+            rule->field = flb_sds_create_len(sentry->value, sentry->len);
+        }
+        else {
+            rule->field = flb_sds_create_size(sentry->len + 2);
+            tmp = flb_sds_cat(rule->field, "$", 1);
+            rule->field = tmp;
+
+            tmp = flb_sds_cat(rule->field, sentry->value, sentry->len);
+            rule->field = tmp;
+        }
 
         /* Get remaining content (regular expression) */
         sentry = mk_list_entry_last(split, struct flb_split_entry, _head);
@@ -104,10 +118,19 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
         /* Release split */
         flb_utils_split_free(split);
 
+        /* Create a record accessor context for this rule */
+        rule->ra = flb_ra_create(rule->field, FLB_FALSE);
+        if (!rule->ra) {
+            flb_plg_error(ctx->ins, "invalid record accessor? '%s'", rule->field);
+            delete_rules(ctx);
+            flb_free(rule);
+            return -1;
+        }
+
         /* Convert string to regex pattern */
         rule->regex = flb_regex_create(rule->regex_pattern);
         if (!rule->regex) {
-            flb_error("[filter_grep] could not compile regex pattern '%s'",
+            flb_plg_error(ctx->ins, "could not compile regex pattern '%s'",
                       rule->regex_pattern);
             delete_rules(ctx);
             flb_free(rule);
@@ -124,14 +147,7 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
 /* Given a msgpack record, do some filter action based on the defined rules */
 static inline int grep_filter_data(msgpack_object map, struct grep_ctx *ctx)
 {
-    int i;
-    int klen;
-    int vlen;
-    const char *key;
-    const char *val;
     ssize_t ret;
-    msgpack_object *k;
-    msgpack_object *v;
     struct mk_list *head;
     struct grep_rule *rule;
 
@@ -139,59 +155,7 @@ static inline int grep_filter_data(msgpack_object map, struct grep_ctx *ctx)
     mk_list_foreach(head, &ctx->rules) {
         rule = mk_list_entry(head, struct grep_rule, _head);
 
-        /* Lookup target key/value */
-        for (i = 0; i < map.via.map.size; i++) {
-            k = &map.via.map.ptr[i].key;
-
-            if (k->type != MSGPACK_OBJECT_BIN &&
-                k->type != MSGPACK_OBJECT_STR) {
-                continue;
-            }
-
-            if (k->type == MSGPACK_OBJECT_STR) {
-                key  = k->via.str.ptr;
-                klen = k->via.str.size;
-            }
-            else {
-                key = k->via.bin.ptr;
-                klen = k->via.bin.size;
-            }
-
-            if (rule->field_len == klen &&
-                strncmp(key, rule->field, klen) == 0) {
-                break;
-            }
-
-            k = NULL;
-        }
-
-        /* If the key don't exists, take an action */
-        if (!k) {
-            if (rule->type == GREP_REGEX) {
-                return GREP_RET_EXCLUDE;
-            }
-            else if (rule->type == GREP_EXCLUDE) {
-                return GREP_RET_KEEP;
-            }
-        }
-
-        /* Based on the value of the key, do the regex */
-        v = &map.via.map.ptr[i].val;
-
-        /* a value must be a string */
-        if (v->type == MSGPACK_OBJECT_STR) {
-            val  = v->via.str.ptr;
-            vlen = v->via.str.size;
-        }
-        else if(v->type == MSGPACK_OBJECT_BIN) {
-            val  = v->via.bin.ptr;
-            vlen = v->via.bin.size;
-        }
-        else {
-            return GREP_RET_EXCLUDE;
-        }
-
-        ret = flb_regex_match(rule->regex,(unsigned char *) val, vlen);
+        ret = flb_ra_regex_match(rule->ra, map, rule->regex, NULL);
         if (ret <= 0) { /* no match */
             if (rule->type == GREP_REGEX) {
                 return GREP_RET_EXCLUDE;
@@ -224,6 +188,7 @@ static int cb_grep_init(struct flb_filter_instance *f_ins,
         return -1;
     }
     mk_list_init(&ctx->rules);
+    ctx->ins = f_ins;
 
     /* Load rules */
     ret = set_rules(ctx, f_ins);
@@ -302,6 +267,10 @@ static int cb_grep_exit(void *data, struct flb_config *config)
 {
     struct grep_ctx *ctx = data;
 
+    if (!ctx) {
+        return 0;
+    }
+
     delete_rules(ctx);
     flb_free(ctx);
     return 0;
