@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,6 +25,7 @@
 #include <fluent-bit/flb_input.h>
 #include <fluent-bit/flb_parser.h>
 #include <fluent-bit/flb_macros.h>
+#include <fluent-bit/flb_sqldb.h>
 #ifdef FLB_HAVE_REGEX
 #include <fluent-bit/flb_regex.h>
 #endif
@@ -55,6 +56,7 @@ struct flb_tail_config {
     /* Collectors */
     int coll_fd_static;
     int coll_fd_scan;
+    int coll_fd_watcher;
     int coll_fd_rotated;
     int coll_fd_pending;
     int coll_fd_dmode_flush;
@@ -72,14 +74,12 @@ struct flb_tail_config {
     int refresh_interval_sec;  /* seconds to re-scan           */
     long refresh_interval_nsec;/* nanoseconds to re-scan       */
     int rotate_wait;           /* sec to wait on rotated files */
+    int watcher_interval;      /* watcher interval             */
     int ignore_older;          /* ignore fields older than X seconds        */
     time_t last_pending;       /* last time a 'pending signal' was emitted' */
-    const char *path;          /* lookup path (glob)           */
-    const char *exclude_path;  /* exclude path                 */
-    const char *path_key;      /* key name of file path        */
-    int   path_key_len;        /* length of key name           */
-    char *key;                 /* key for unstructured record  */
-    int   key_len;             /* length of key ^              */
+    struct mk_list *path_list; /* list of paths to scan (glob) */
+    flb_sds_t path_key;        /* key name of file path        */
+    flb_sds_t key;             /* key for unstructured record  */
     int   skip_long_lines;     /* skip long lines              */
     int   exit_on_eof;         /* exit fluent-bit on EOF, test */
 
@@ -87,6 +87,7 @@ struct flb_tail_config {
 #ifdef FLB_HAVE_SQLDB
     struct flb_sqldb *db;
     int db_sync;
+    sqlite3_stmt *stmt_offset;
 #endif
 
     /* Parser / Format */
@@ -101,6 +102,7 @@ struct flb_tail_config {
     /* Docker mode */
     int docker_mode;           /* Docker mode enabled ?  */
     int docker_mode_flush;     /* Docker mode flush/wait */
+    struct flb_parser *docker_mode_parser; /* Parser for separate multiline logs */
 
     /* Lists head for files consumed statically (read) and by events (inotify) */
     struct mk_list files_static;
@@ -113,10 +115,10 @@ struct flb_tail_config {
     struct mk_list *exclude_list;
 
     /* Plugin input instance */
-    struct flb_input_instance *i_ins;
+    struct flb_input_instance *ins;
 };
 
-struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *i_ins,
+struct flb_tail_config *flb_tail_config_create(struct flb_input_instance *ins,
                                                struct flb_config *config);
 int flb_tail_config_destroy(struct flb_tail_config *config);
 
