@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,7 +25,7 @@
 
 #include <fluent-bit/flb_compat.h>
 #include <fluent-bit/flb_info.h>
-#include <fluent-bit/flb_input.h>
+#include <fluent-bit/flb_input_plugin.h>
 #include <fluent-bit/flb_parser.h>
 #ifdef FLB_HAVE_REGEX
 #include <fluent-bit/flb_regex.h>
@@ -41,26 +41,8 @@
 #include "tail_multiline.h"
 #include "tail_scan.h"
 
-#ifdef _MSC_VER
-static int get_inode(int fd, uint64_t *inode)
-{
-    HANDLE h;
-    BY_HANDLE_FILE_INFORMATION info;
-
-    h = _get_osfhandle(fd);
-    if (h == INVALID_HANDLE_VALUE) {
-        flb_error("[in_tail] cannot convert fd:%i into HANDLE", fd);
-        return -1;
-    }
-
-    if (GetFileInformationByHandle(h, &info) == 0) {
-        flb_error("[in_tail] cannot get file info for fd:%i", fd);
-        return -1;
-    }
-    *inode = (uint64_t) info.nFileIndexHigh;
-    *inode = *inode << 32 | info.nFileIndexLow;
-    return 0;
-}
+#ifdef FLB_SYSTEM_WINDOWS
+#include "win32.h"
 #endif
 
 static inline void consume_bytes(char *buf, int bytes, int length)
@@ -68,28 +50,6 @@ static inline void consume_bytes(char *buf, int bytes, int length)
     memmove(buf, buf + bytes, length - bytes);
 }
 
-#ifdef _MSC_VER
-/*
- * Open a file for reading. We need to use CreateFileA() instead of
- * open(2) to avoid automatic file locking.
- */
-static int open_file(const char *path)
-{
-    HANDLE h;
-    h = CreateFileA(path,
-                    GENERIC_READ,
-                    FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
-                    NULL,           /* lpSecurityAttributes */
-                    OPEN_EXISTING,  /* dwCreationDisposition */
-                    0,              /* dwFlagsAndAttributes */
-                    NULL);          /* hTemplateFile */
-    if (h == INVALID_HANDLE_VALUE) {
-        return -1;
-    }
-    return _open_osfhandle((intptr_t) h, _O_RDONLY);
-}
-#endif
-
 static int unpack_and_pack(msgpack_packer *pck, msgpack_object *root,
                            const char *key, size_t key_len,
                            const char *val, size_t val_len)
@@ -171,7 +131,7 @@ int flb_tail_pack_line_map(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
     if (file->config->path_key != NULL) {
         append_record_to_map(data, data_size,
                              file->config->path_key,
-                             file->config->path_key_len,
+                             flb_sds_len(file->config->path_key),
                              file->name, file->name_len);
     }
 
@@ -198,15 +158,15 @@ int flb_tail_file_pack_line(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
 
     if (file->config->path_key != NULL) {
         /* append path_key */
-        msgpack_pack_str(mp_pck, file->config->path_key_len);
+        msgpack_pack_str(mp_pck, flb_sds_len(file->config->path_key));
         msgpack_pack_str_body(mp_pck, file->config->path_key,
-                              file->config->path_key_len);
+                              flb_sds_len(file->config->path_key));
         msgpack_pack_str(mp_pck, file->name_len);
         msgpack_pack_str_body(mp_pck, file->name, file->name_len);
     }
 
-    msgpack_pack_str(mp_pck, ctx->key_len);
-    msgpack_pack_str_body(mp_pck, ctx->key, ctx->key_len);
+    msgpack_pack_str(mp_pck, flb_sds_len(ctx->key));
+    msgpack_pack_str_body(mp_pck, ctx->key, flb_sds_len(ctx->key));
     msgpack_pack_str(mp_pck, data_size);
     msgpack_pack_str_body(mp_pck, data, data_size);
 
@@ -281,7 +241,7 @@ static int process_content(struct flb_tail_file *file, off_t *bytes)
         if (ctx->docker_mode) {
             ret = flb_tail_dmode_process_content(now, line, line_len,
                                                  &repl_line, &repl_line_len,
-                                                 file, ctx);
+                                                 file, ctx, out_sbuf, out_pck);
             if (ret >= 0) {
                 if (repl_line == line) {
                     repl_line = NULL;
@@ -290,9 +250,8 @@ static int process_content(struct flb_tail_file *file, off_t *bytes)
                     line = repl_line;
                     line_len = repl_line_len;
                 }
-                if (ret == 0) {
-                    goto go_next;
-                }
+                /* Skip normal parsers flow */
+                goto go_next;
             }
             else {
                 flb_tail_dmode_flush(out_sbuf, out_pck, file, ctx);
@@ -377,7 +336,7 @@ static int process_content(struct flb_tail_file *file, off_t *bytes)
     *bytes = processed_bytes;
 
     /* Append buffer content to a chunk */
-    flb_input_chunk_append_raw(ctx->i_ins,
+    flb_input_chunk_append_raw(ctx->ins,
                                file->tag_buf,
                                file->tag_len,
                                out_sbuf->data,
@@ -409,9 +368,12 @@ static void cb_results(const char *name, const char *value,
 #endif
 
 #ifdef FLB_HAVE_REGEX
-static int tag_compose(char *tag, struct flb_regex *tag_regex, char *fname, char *out_buf, size_t *out_size)
+static int tag_compose(char *tag, struct flb_regex *tag_regex, char *fname,
+                       char *out_buf, size_t *out_size,
+                       struct flb_tail_config *ctx)
 #else
-static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
+static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size,
+                       struct flb_tail_config *ctx)
 #endif
 {
     int i;
@@ -433,7 +395,8 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
     if (tag_regex) {
         n = flb_regex_do(tag_regex, fname, strlen(fname), &result);
         if (n <= 0) {
-            flb_error("[in_tail] invalid pattern for given file %s", fname);
+            flb_plg_error(ctx->ins, "invalid tag_regex pattern for file %s",
+                          fname);
             return -1;
         }
         else {
@@ -465,7 +428,9 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
                     }
                 }
                 else {
-                    flb_error("[in_tail] missing closing angle bracket in tag %s at position %i", tag, beg - tag);
+                    flb_plg_error(ctx->ins,
+                                  "missing closing angle bracket in tag %s "
+                                  "at position %lu", tag, beg - tag);
                     flb_hash_destroy(ht);
                     return -1;
                 }
@@ -509,7 +474,7 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
 
         /* Sanitize buffer */
         for (i = 0; i < buf_s; i++) {
-            if (out_buf[i] == '/') {
+            if (out_buf[i] == '/' || out_buf[i] == '\\' || out_buf[i] == ':') {
                 if (i > 0) {
                     out_buf[i] = '.';
                 }
@@ -520,7 +485,7 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
                 }
             }
 
-            if (out_buf[i] == '.' && i > 0) {
+            if (i > 0 && out_buf[i] == '.') {
                 if (out_buf[i - 1] == '.') {
                     drop_bytes(out_buf, buf_s, i, 1);
                     buf_s--;
@@ -549,7 +514,8 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size)
     return 0;
 }
 
-int flb_tail_file_exists(char *name, struct flb_tail_config *ctx)
+static inline int flb_tail_file_exists(struct stat *st,
+                                       struct flb_tail_config *ctx)
 {
     struct mk_list *head;
     struct flb_tail_file *file;
@@ -557,7 +523,7 @@ int flb_tail_file_exists(char *name, struct flb_tail_config *ctx)
     /* Iterate static list */
     mk_list_foreach(head, &ctx->files_static) {
         file = mk_list_entry(head, struct flb_tail_file, _head);
-        if (flb_tail_file_name_cmp(name, file) == 0) {
+        if (file->inode == st->st_ino) {
             return FLB_TRUE;
         }
     }
@@ -565,7 +531,7 @@ int flb_tail_file_exists(char *name, struct flb_tail_config *ctx)
     /* Iterate dynamic list */
     mk_list_foreach(head, &ctx->files_event) {
         file = mk_list_entry(head, struct flb_tail_file, _head);
-        if (flb_tail_file_name_cmp(name, file) == 0) {
+        if (file->inode == st->st_ino) {
             return FLB_TRUE;
         }
     }
@@ -582,35 +548,20 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     off_t offset;
     char *tag;
     size_t tag_len;
-    struct mk_list *head;
     struct flb_tail_file *file;
 
     if (!S_ISREG(st->st_mode)) {
         return -1;
     }
 
-    /* Double check this file is not already being monitored */
-    mk_list_foreach(head, &ctx->files_static) {
-        file = mk_list_entry(head, struct flb_tail_file, _head);
-        if (flb_tail_file_name_cmp(path, file) == 0) {
-            return -1;
-        }
-    }
-    mk_list_foreach(head, &ctx->files_event) {
-        file = mk_list_entry(head, struct flb_tail_file, _head);
-        if (flb_tail_file_name_cmp(path, file) == 0) {
-            return -1;
-        }
+    if (flb_tail_file_exists(st, ctx) == FLB_TRUE) {
+        return -1;
     }
 
-#ifdef _MSC_VER
-    fd = open_file(path);
-#else
     fd = open(path, O_RDONLY);
-#endif
     if (fd == -1) {
         flb_errno();
-        flb_error("[in_tail] could not open %s", path);
+        flb_plg_error(ctx->ins, "cannot open %s", path);
         return -1;
     }
 
@@ -624,6 +575,16 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     file->watch_fd  = -1;
     file->fd        = fd;
 
+    /* On non-windows environments check if the original path is a link */
+    struct stat lst;
+    ret = lstat(path, &lst);
+    if (ret == 0) {
+        if (S_ISLNK(lst.st_mode)) {
+            file->is_link = FLB_TRUE;
+            file->link_inode = lst.st_ino;
+        }
+    }
+
     /*
      * Duplicate string into 'file' structure, the called function
      * take cares to resolve real-name of the file in case we are
@@ -641,25 +602,7 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
         goto error;
     }
 
-#if defined(__APPLE__)
-    char *name = flb_tail_file_name(file);
-    if (name == NULL) {
-        goto error;
-    }
-    if (flb_tail_file_exists(name, ctx)) {
-        flb_free(name);
-        goto error;
-    }
-    flb_free(name);
-#endif
-
-#ifdef _MSC_VER
-    if (get_inode(fd, &file->inode)) {
-        goto error;
-    }
-#else
     file->inode     = st->st_ino;
-#endif
     file->offset    = 0;
     file->size      = st->st_size;
     file->buf_len   = 0;
@@ -676,6 +619,7 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     file->mult_skipping = FLB_FALSE;
     msgpack_sbuffer_init(&file->mult_sbuf);
     file->dmode_flush_timeout = 0;
+    file->dmode_complete = true;
     file->dmode_buf = flb_sds_create_size(ctx->docker_mode == FLB_TRUE ? 65536 : 0);
     file->dmode_lastline = flb_sds_create_size(ctx->docker_mode == FLB_TRUE ? 20000 : 0);
 #ifdef FLB_HAVE_SQLDB
@@ -694,17 +638,17 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
 
     /* Initialize (optional) dynamic tag */
     if (ctx->dynamic_tag == FLB_TRUE) {
-        len = ctx->i_ins->tag_len + strlen(path) + 1;
+        len = ctx->ins->tag_len + strlen(path) + 1;
         tag = flb_malloc(len);
         if (!tag) {
             flb_errno();
-            flb_error("[in_tail] failed to allocate tag buffer");
+            flb_plg_error(ctx->ins, "failed to allocate tag buffer");
             goto error;
         }
 #ifdef FLB_HAVE_REGEX
-        ret = tag_compose(ctx->i_ins->tag, ctx->tag_regex, path, tag, &tag_len);
+        ret = tag_compose(ctx->ins->tag, ctx->tag_regex, path, tag, &tag_len, ctx);
 #else
-        ret = tag_compose(ctx->i_ins->tag, path, tag, &tag_len);
+        ret = tag_compose(ctx->ins->tag, path, tag, &tag_len, ctx);
 #endif
         if (ret == 0) {
             file->tag_len = tag_len;
@@ -712,32 +656,33 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
         }
         flb_free(tag);
         if (ret != 0) {
-            flb_error("[in_tail] failed to compose tag for file: %s", path);
+            flb_plg_error(ctx->ins, "failed to compose tag for file: %s", path);
             goto error;
         }
     }
     else {
-        file->tag_len = strlen(ctx->i_ins->tag);
-        file->tag_buf = flb_strdup(ctx->i_ins->tag);
+        file->tag_len = strlen(ctx->ins->tag);
+        file->tag_buf = flb_strdup(ctx->ins->tag);
     }
     if (!file->tag_buf) {
-        flb_error("[in_tail] failed to set tag for file: %s", path);
+        flb_plg_error(ctx->ins, "failed to set tag for file: %s", path);
         flb_errno();
         goto error;
     }
 
-    /* Register this file into the fs_event monitoring */
-    ret = flb_tail_fs_add(file);
-    if (ret == -1) {
-        flb_error("[in_tail] could not register file into fs_events");
-        goto error;
-    }
-
     if (mode == FLB_TAIL_STATIC) {
         mk_list_add(&file->_head, &ctx->files_static);
+        tail_signal_manager(file->config);
     }
     else if (mode == FLB_TAIL_EVENT) {
         mk_list_add(&file->_head, &ctx->files_event);
+
+        /* Register this file into the fs_event monitoring */
+        ret = flb_tail_fs_add(file);
+        if (ret == -1) {
+            flb_plg_error(ctx->ins, "could not register file into fs_events");
+            goto error;
+        }
     }
 
     /*
@@ -752,6 +697,8 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
 
     /* Seek if required */
     if (file->offset > 0) {
+        flb_plg_debug(ctx->ins, "inode=%"PRIu64" appended file following on offset=%lu",
+                      file->inode, file->offset);
         offset = lseek(file->fd, file->offset, SEEK_SET);
         if (offset == -1) {
             flb_errno();
@@ -760,11 +707,13 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
         }
     }
 
+    file->pending_bytes = file->size - file->offset;
+
 #ifdef FLB_HAVE_METRICS
-    flb_metrics_sum(FLB_TAIL_METRIC_F_OPENED, 1, ctx->i_ins->metrics);
+    flb_metrics_sum(FLB_TAIL_METRIC_F_OPENED, 1, ctx->ins->metrics);
 #endif
 
-    flb_debug("[in_tail] add to scan queue %s, offset=%lu", path, file->offset);
+    flb_plg_debug(ctx->ins, "inode=%"PRIu64" appended as %s", file->inode, path);
     return 0;
 
 error:
@@ -787,6 +736,10 @@ void flb_tail_file_remove(struct flb_tail_file *file)
     struct flb_tail_config *ctx;
 
     ctx = file->config;
+
+    flb_plg_debug(ctx->ins, "inode=%"PRIu64" removing file name %s",
+                  file->inode, file->name);
+
     if (file->rotated > 0) {
 #ifdef FLB_HAVE_SQLDB
         /*
@@ -811,12 +764,10 @@ void flb_tail_file_remove(struct flb_tail_file *file)
 
     flb_free(file->buf_data);
     flb_free(file->name);
-#if !defined(__linux__)
     flb_free(file->real_name);
-#endif
 
 #ifdef FLB_HAVE_METRICS
-    flb_metrics_sum(FLB_TAIL_METRIC_F_CLOSED, 1, ctx->i_ins->metrics);
+    flb_metrics_sum(FLB_TAIL_METRIC_F_CLOSED, 1, ctx->ins->metrics);
 #endif
 
     flb_free(file);
@@ -852,11 +803,12 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
     off_t capacity;
     off_t processed_bytes;
     ssize_t bytes;
+    struct stat st;
     struct flb_tail_config *ctx;
 
     /* Check if we the engine issued a pause */
     ctx = file->config;
-    if (flb_input_buf_paused(ctx->i_ins) == FLB_TRUE) {
+    if (flb_input_buf_paused(ctx->ins) == FLB_TRUE) {
         return FLB_TAIL_BUSY;
     }
 
@@ -868,15 +820,15 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
          */
         if (file->buf_size >= ctx->buf_max_size) {
             if (ctx->skip_long_lines == FLB_FALSE) {
-                flb_error("[in_tail] file=%s requires a larger buffer size, "
+                flb_plg_error(ctx->ins, "file=%s requires a larger buffer size, "
                           "lines are too long. Skipping file.", file->name);
                 return FLB_TAIL_ERROR;
             }
 
             /* Warn the user */
             if (file->skip_warn == FLB_FALSE) {
-                flb_warn("[in_tail] file=%s have long lines. "
-                         "Skipping long lines.", file->name);
+                flb_plg_warn(ctx->ins, "file=%s have long lines. "
+                             "Skipping long lines.", file->name);
                 file->skip_warn = FLB_TRUE;
             }
 
@@ -893,14 +845,15 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
             /* Increase the buffer size */
             tmp = flb_realloc(file->buf_data, size);
             if (tmp) {
-                flb_trace("[in_tail] file=%s increase buffer size %lu => %lu bytes",
-                          file->name, file->buf_size, size);
+                flb_plg_trace(ctx->ins, "file=%s increase buffer size "
+                              "%lu => %lu bytes",
+                              file->name, file->buf_size, size);
                 file->buf_data = tmp;
                 file->buf_size = size;
             }
             else {
                 flb_errno();
-                flb_error("[in_tail] cannot increase buffer size for %s, "
+                flb_plg_error(ctx->ins, "cannot increase buffer size for %s, "
                           "skipping file.", file->name);
                 return FLB_TAIL_ERROR;
             }
@@ -922,12 +875,9 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
          * line.
          */
         ret = process_content(file, &processed_bytes);
-        if (ret >= 0) {
-            flb_debug("[in_tail] file=%s read=%lu lines=%i",
-                      file->name, bytes, ret);
-        }
-        else {
-            flb_debug("[in_tail] file=%s ERROR", file->name);
+        if (ret < 0) {
+            flb_plg_debug(ctx->ins, "inode=%"PRIu64" file=%s process content ERROR",
+                          file->inode, file->name);
             return FLB_TAIL_ERROR;
         }
 
@@ -944,33 +894,125 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
         }
 #endif
 
+        ret = fstat(file->fd, &st);
+        if (ret == -1) {
+            flb_errno();
+            return FLB_TAIL_ERROR;
+        }
+        else {
+            file->size = st.st_size;
+            file->pending_bytes = (st.st_size - file->offset);
+        }
         /* Data was consumed but likely some bytes still remain */
         return FLB_TAIL_OK;
     }
     else if (bytes == 0) {
         /* We reached the end of file, let's wait for some incoming data */
+        ret = fstat(file->fd, &st);
+        if (ret == -1) {
+            flb_errno();
+            return FLB_TAIL_ERROR;
+        }
+        file->size = st.st_size;
+        file->pending_bytes = (st.st_size - file->offset);
+
         return FLB_TAIL_WAIT;
     }
     else {
         /* error */
         flb_errno();
-        flb_error("[in_tail] error reading %s", file->name);
+        flb_plg_error(ctx->ins, "error reading %s", file->name);
         return FLB_TAIL_ERROR;
     }
 
     return FLB_TAIL_ERROR;
 }
 
-int flb_tail_file_to_event(struct flb_tail_file *file)
+/* Returns FLB_TRUE if a file has been rotated, otherwise FLB_FALSE */
+int flb_tail_file_is_rotated(struct flb_tail_config *ctx,
+                             struct flb_tail_file *file)
 {
     int ret;
     char *name;
     struct stat st;
-    struct stat st_rotated;
+
+    /*
+     * Do not double-check already rotated files since the caller of this
+     * function will trigger a rotation.
+     */
+    if (file->rotated != 0) {
+        return FLB_FALSE;
+    }
+
+    /* Check if the 'original monitored file' is a link and rotated */
+    if (file->is_link == FLB_TRUE) {
+        ret = lstat(file->name, &st);
+        if (ret == -1) {
+            /* Broken link or missing file */
+            if (errno == ENOENT) {
+                flb_plg_info(ctx->ins, "inode=%"PRIu64" link_rotated: %s",
+                             file->link_inode, file->name);
+                return FLB_TRUE;
+            }
+            else {
+                flb_errno();
+                flb_plg_error(ctx->ins,
+                              "link_inode=%"PRIu64" cannot detect if file: %s",
+                              file->link_inode, file->name);
+                return -1;
+            }
+        }
+        else {
+            /* The file name is there, check if the same that we have */
+            if (st.st_ino != file->link_inode) {
+                return FLB_TRUE;
+            }
+        }
+    }
+
+    /* Retrieve the real file name, operating system lookup */
+    name = flb_tail_file_name(file);
+    if (!name) {
+        flb_plg_error(ctx->ins,
+                      "inode=%"PRIu64" cannot detect if file was rotated: %s",
+                      file->inode, file->name);
+        return -1;
+    }
+
+
+    /* Get stats from the file name */
+    ret = stat(name, &st);
+    if (ret == -1) {
+        flb_errno();
+        flb_free(name);
+        return -1;
+    }
+
+    /* Compare inodes and names */
+    if (file->inode == st.st_ino &&
+        flb_tail_target_file_name_cmp(name, file) == 0) {
+        flb_free(name);
+        return FLB_FALSE;
+    }
+
+    flb_plg_debug(ctx->ins, "inode=%"PRIu64" rotated: %s => %s",
+                  file->inode, file->name, name);
+
+    flb_free(name);
+    return FLB_TRUE;
+}
+
+/* Promote a event in the static list to the dynamic 'events' interface */
+int flb_tail_file_to_event(struct flb_tail_file *file)
+{
+    int ret;
+    struct stat st;
+    struct flb_tail_config *ctx = file->config;
 
     /* Check if the file promoted have pending bytes */
     ret = fstat(file->fd, &st);
     if (ret != 0) {
+        flb_errno();
         return -1;
     }
 
@@ -982,28 +1024,12 @@ int flb_tail_file_to_event(struct flb_tail_file *file)
         file->pending_bytes = 0;
     }
 
-    /* Check if this file have been rotated */
-    name = flb_tail_file_name(file);
-    if (!name) {
-        flb_debug("[in_tail] cannot detect if file was rotated: %s",
-                  file->name);
-        return -1;
+    /* Check if the file has been rotated */
+    ret = flb_tail_file_is_rotated(ctx, file);
+    if (ret == FLB_TRUE) {
+        flb_tail_file_rotated(file);
     }
 
-    if (flb_tail_file_name_cmp(name, file) != 0) {
-        ret = stat(name, &st_rotated);
-        if (ret == -1) {
-            flb_free(name);
-            return -1;
-        }
-        else if (st_rotated.st_ino != st.st_ino) {
-            flb_trace("[in_tail] static file rotated: %s => to %s",
-                      file->name, name);
-            flb_tail_file_rotated(file);
-        }
-    }
-    flb_free(name);
-
     /* Notify the fs-event handler that we will start monitoring this 'file' */
     ret = flb_tail_fs_add(file);
     if (ret == -1) {
@@ -1106,7 +1132,6 @@ int flb_tail_file_name_dup(char *path, struct flb_tail_file *file)
     }
     file->name_len = strlen(file->name);
 
-#if !defined(__linux__)
     if (file->real_name) {
         flb_free(file->real_name);
     }
@@ -1117,7 +1142,6 @@ int flb_tail_file_name_dup(char *path, struct flb_tail_file *file)
         file->name = NULL;
         return -1;
     }
-#endif
 
     return 0;
 }
@@ -1126,59 +1150,58 @@ int flb_tail_file_name_dup(char *path, struct flb_tail_file *file)
 int flb_tail_file_rotated(struct flb_tail_file *file)
 {
     int ret;
-    int create = FLB_FALSE;
     char *name;
     char *tmp;
     struct stat st;
     struct flb_tail_config *ctx = file->config;
 
-    /* Get stats from the original file name (if a new one exists) */
-    ret = stat(file->name, &st);
-    if (ret == 0) {
-        /* Check if we need to re-create an entry with the original name */
-        if (st.st_ino != file->inode && file->rotated == 0) {
-            create = FLB_TRUE;
-        }
-    }
-
-#ifdef FLB_HAVE_METRICS
-    flb_metrics_sum(FLB_TAIL_METRIC_F_ROTATED,
-                    1, file->config->i_ins->metrics);
-#endif
-
     /* Get the new file name */
     name = flb_tail_file_name(file);
     if (!name) {
         return -1;
     }
 
-    flb_debug("[in_tail] rotated: %s -> %s",
-              file->name, name);
-
-    /* Rotate the file in the database */
-#ifdef FLB_HAVE_SQLDB
-    if (file->config->db) {
-        ret = flb_tail_db_file_rotate(name, file, file->config);
-        if (ret == -1) {
-            flb_error("[in_tail] could not rotate file %s->%s in database",
-                      file->name, name);
-        }
-    }
-#endif
+    flb_plg_debug(ctx->ins, "inode=%"PRIu64" rotated %s -> %s",
+                  file->inode, file->name, name);
 
     /* Update local file entry */
-    tmp        = file->name;
+    tmp = file->name;
     flb_tail_file_name_dup(name, file);
-
+    flb_plg_info(ctx->ins, "inode=%"PRIu64" handle rotation(): %s => %s",
+                 file->inode, tmp, file->name);
     if (file->rotated == 0) {
         file->rotated = time(NULL);
         mk_list_add(&file->_rotate_head, &file->config->files_rotated);
-    }
 
-    /* Request to append 'new' file created */
-    if (create == FLB_TRUE) {
-        flb_tail_scan(ctx->path, ctx);
-        tail_signal_manager(file->config);
+    /* Rotate the file in the database */
+#ifdef FLB_HAVE_SQLDB
+        if (file->config->db) {
+            ret = flb_tail_db_file_rotate(name, file, file->config);
+            if (ret == -1) {
+                flb_plg_error(ctx->ins, "could not rotate file %s->%s in database",
+                              file->name, name);
+            }
+        }
+#endif
+
+#ifdef FLB_HAVE_METRICS
+        flb_metrics_sum(FLB_TAIL_METRIC_F_ROTATED,
+                        1, file->config->ins->metrics);
+#endif
+
+        /* Check if a new file has been created */
+        ret = stat(tmp, &st);
+        if (ret == 0 && st.st_ino != file->inode) {
+            if (flb_tail_file_exists(&st, ctx) == FLB_FALSE) {
+                ret = flb_tail_file_append(tmp, &st, FLB_TAIL_STATIC, ctx);
+                if (ret == -1) {
+                    flb_tail_scan(ctx->path_list, ctx);
+                }
+                else {
+                    tail_signal_manager(file->config);
+                }
+            }
+        }
     }
     flb_free(tmp);
     flb_free(name);
@@ -1186,28 +1209,91 @@ int flb_tail_file_rotated(struct flb_tail_file *file)
     return 0;
 }
 
-int flb_tail_file_rotated_purge(struct flb_input_instance *i_ins,
-                                struct flb_config *config, void *context)
+static int check_purge_deleted_file(struct flb_tail_config *ctx,
+                                    struct flb_tail_file *file)
 {
+    int ret;
+    struct stat st;
+
+    ret = fstat(file->fd, &st);
+    if (ret == -1) {
+        flb_plg_debug(ctx->ins, "error stat(2) %s, removing", file->name);
+        flb_tail_file_remove(file);
+        return FLB_TRUE;
+    }
+
+    if (st.st_nlink == 0) {
+        flb_plg_debug(ctx->ins, "purge: monitored file has been deleted: %s",
+                      file->name);
+#ifdef FLB_HAVE_SQLDB
+        if (ctx->db) {
+            /* Remove file entry from the database */
+            flb_tail_db_file_delete(file, file->config);
+        }
+#endif
+        /* Remove file from the monitored list */
+        flb_tail_file_remove(file);
+        return FLB_TRUE;
+    }
+
+    return FLB_FALSE;
+}
+
+/* Purge rotated and deleted files */
+int flb_tail_file_purge(struct flb_input_instance *ins,
+                        struct flb_config *config, void *context)
+{
+    int ret;
     int count = 0;
     struct mk_list *tmp;
     struct mk_list *head;
     struct flb_tail_file *file;
     struct flb_tail_config *ctx = context;
     time_t now;
+    struct stat st;
 
+    /* Rotated files */
     now = time(NULL);
     mk_list_foreach_safe(head, tmp, &ctx->files_rotated) {
         file = mk_list_entry(head, struct flb_tail_file, _rotate_head);
         if ((file->rotated + ctx->rotate_wait) <= now) {
-            flb_debug("[in_tail] purge rotated file %s", file->name);
-            if (file->pending_bytes > 0 && flb_input_buf_paused(i_ins)) {
-                flb_warn("[in_tail] purged rotated file while data ingestion is paused, consider increasing rotate_wait");
+            ret = fstat(file->fd, &st);
+            if (ret == 0) {
+                flb_plg_debug(ctx->ins,
+                              "inode=%"PRIu64" purge rotated file %s " \
+                              "(offset=%lu / size = %"PRIu64")",
+                              file->inode, file->name, file->offset, st.st_size);
+                if (file->pending_bytes > 0 && flb_input_buf_paused(ins)) {
+                    flb_plg_warn(ctx->ins, "purged rotated file while data "
+                                 "ingestion is paused, consider increasing "
+                                 "rotate_wait");
+                }
             }
+            else {
+                flb_plg_debug(ctx->ins,
+                              "inode=%"PRIu64" purge rotated file %s (offset=%lu)",
+                              file->inode, file->name, file->offset);
+            }
+
             flb_tail_file_remove(file);
             count++;
         }
     }
 
+    /*
+     * Deleted files: under high load scenarios, exists the chances that in
+     * our event loop we miss some notifications about a file. In order to
+     * sanitize our list of monitored files we will iterate all of them and check
+     * if they have been deleted or not.
+     */
+    mk_list_foreach_safe(head, tmp, &ctx->files_static) {
+        file = mk_list_entry(head, struct flb_tail_file, _head);
+        check_purge_deleted_file(ctx, file);
+    }
+    mk_list_foreach_safe(head, tmp, &ctx->files_event) {
+        file = mk_list_entry(head, struct flb_tail_file, _head);
+        check_purge_deleted_file(ctx, file);
+    }
+
     return count;
 }
