@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,6 +19,7 @@
  */
 
 #include <fluent-bit/flb_info.h>
+#include <fluent-bit/flb_input_plugin.h>
 #include <fluent-bit/flb_config.h>
 #include <fluent-bit/flb_kv.h>
 
@@ -43,7 +44,7 @@ static int tail_mult_append(struct flb_parser *parser,
 }
 
 int flb_tail_mult_create(struct flb_tail_config *ctx,
-                         struct flb_input_instance *i_ins,
+                         struct flb_input_instance *ins,
                          struct flb_config *config)
 {
     int ret;
@@ -52,26 +53,19 @@ int flb_tail_mult_create(struct flb_tail_config *ctx,
     struct flb_parser *parser;
     struct flb_kv *kv;
 
-    tmp = flb_input_get_property("multiline_flush", i_ins);
-    if (!tmp) {
-        ctx->multiline_flush = FLB_TAIL_MULT_FLUSH;
-    }
-    else {
-        ctx->multiline_flush = atoi(tmp);
-        if (ctx->multiline_flush <= 0) {
-            ctx->multiline_flush = 1;
-        }
+    if (ctx->multiline_flush <= 0) {
+        ctx->multiline_flush = 1;
     }
 
     /* Get firstline parser */
-    tmp = flb_input_get_property("parser_firstline", i_ins);
+    tmp = flb_input_get_property("parser_firstline", ins);
     if (!tmp) {
-        flb_error("[in_tail] No parser defined for firstline");
+        flb_plg_error(ctx->ins, "multiline: no parser defined for firstline");
         return -1;
     }
     parser = flb_parser_get(tmp, config);
     if (!parser) {
-        flb_error("[in_tail] multiline: invalid parser '%s'", tmp);
+        flb_plg_error(ctx->ins, "multiline: invalid parser '%s'", tmp);
         return -1;
     }
 
@@ -79,7 +73,7 @@ int flb_tail_mult_create(struct flb_tail_config *ctx,
     mk_list_init(&ctx->mult_parsers);
 
     /* Read all multiline rules */
-    mk_list_foreach(head, &i_ins->properties) {
+    mk_list_foreach(head, &ins->properties) {
         kv = mk_list_entry(head, struct flb_kv, _head);
         if (strcasecmp("parser_firstline", kv->key) == 0) {
             continue;
@@ -88,7 +82,7 @@ int flb_tail_mult_create(struct flb_tail_config *ctx,
         if (strncasecmp("parser_", kv->key, 7) == 0) {
             parser = flb_parser_get(kv->val, config);
             if (!parser) {
-                flb_error("[in_tail] multiline: invalid parser '%s'", kv->val);
+                flb_plg_error(ctx->ins, "multiline: invalid parser '%s'", kv->val);
                 return -1;
             }
 
@@ -137,7 +131,7 @@ static int pack_line(char *data, size_t data_size, struct flb_tail_file *file,
     flb_time_get(&out_time);
 
     flb_tail_file_pack_line(&mp_sbuf, &mp_pck, &out_time, data, data_size, file);
-    flb_input_chunk_append_raw(ctx->i_ins,
+    flb_input_chunk_append_raw(ctx->ins,
                                file->tag_buf,
                                file->tag_len,
                                mp_sbuf.data,
@@ -167,7 +161,7 @@ int flb_tail_mult_process_first(time_t now,
         msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
 
         flb_tail_mult_flush(&mp_sbuf, &mp_pck, file, ctx);
-        flb_input_chunk_append_raw(ctx->i_ins,
+        flb_input_chunk_append_raw(ctx->ins,
                                    file->tag_buf,
                                    file->tag_len,
                                    mp_sbuf.data,
@@ -259,9 +253,19 @@ static inline int is_last_key_val_string(char *buf, size_t size)
     }
 
     root = result.data;
-    v = root.via.map.ptr[root.via.map.size - 1].val;
-    if (v.type == MSGPACK_OBJECT_STR) {
-        ret = FLB_TRUE;
+    if (root.type != MSGPACK_OBJECT_MAP) {
+        ret = FLB_FALSE;
+    }
+    else {
+        if (root.via.map.size == 0) {
+            ret = FLB_FALSE;
+        }
+        else {
+            v = root.via.map.ptr[root.via.map.size - 1].val;
+            if (v.type == MSGPACK_OBJECT_STR) {
+                ret = FLB_TRUE;
+            }
+        }
     }
 
     msgpack_unpacked_destroy(&result);
@@ -295,12 +299,14 @@ int flb_tail_mult_process_content(time_t now,
          * will be possible.
          */
         ret = is_last_key_val_string(out_buf, out_size);
-        if (ret == FLB_TRUE) {
-            flb_tail_mult_process_first(now, out_buf, out_size, &out_time,
-                                        file, ctx);
-            return FLB_TAIL_MULT_MORE;
-        }
-        flb_free(out_buf);
+        if (ret == FLB_TRUE)
+            file->mult_firstline_append = FLB_TRUE;
+        else
+            file->mult_firstline_append = FLB_FALSE;
+
+        flb_tail_mult_process_first(now, out_buf, out_size, &out_time,
+                                    file, ctx);
+        return FLB_TAIL_MULT_MORE;
     }
 
     if (file->mult_skipping == FLB_TRUE) {
@@ -335,7 +341,7 @@ int flb_tail_mult_process_content(time_t now,
          * If no parser was found means the string log must be appended
          * to the last structured field.
          */
-        if (file->mult_firstline == FLB_TRUE) {
+        if (file->mult_firstline == FLB_TRUE && file->mult_firstline_append == FLB_TRUE) {
             flb_tail_mult_append_raw(buf, len, file, ctx);
         }
         else {
@@ -398,9 +404,9 @@ int flb_tail_mult_flush(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
 
     /* Append Path_Key ? */
     if (file->config->path_key != NULL) {
-        msgpack_pack_str(mp_pck, file->config->path_key_len);
+        msgpack_pack_str(mp_pck, flb_sds_len(file->config->path_key));
         msgpack_pack_str_body(mp_pck, file->config->path_key,
-                              file->config->path_key_len);
+                              flb_sds_len(file->config->path_key));
         msgpack_pack_str(mp_pck, file->name_len);
         msgpack_pack_str_body(mp_pck, file->name, file->name_len);
     }
@@ -485,7 +491,7 @@ int flb_tail_mult_flush(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
     return 0;
 }
 
-int flb_tail_mult_pending_flush(struct flb_input_instance *i_ins,
+int flb_tail_mult_pending_flush(struct flb_input_instance *ins,
                                 struct flb_config *config, void *context)
 {
     time_t now;
@@ -516,7 +522,7 @@ int flb_tail_mult_pending_flush(struct flb_input_instance *i_ins,
 
         flb_tail_mult_flush(&mp_sbuf, &mp_pck, file, ctx);
 
-        flb_input_chunk_append_raw(i_ins,
+        flb_input_chunk_append_raw(ins,
                                    file->tag_buf,
                                    file->tag_len,
                                    mp_sbuf.data,
