@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -46,6 +46,22 @@
 /* Default Resource type */
 #define FLB_SDS_RESOURCE_TYPE "global"
 
+#define OPERATION_FIELD_IN_JSON "logging.googleapis.com/operation"
+#define LOCAL_RESOURCE_ID_KEY "logging.googleapis.com/local_resource_id"
+#define DEFAULT_LABELS_KEY "logging.googleapis.com/labels"
+#define LEN_LOCAL_RESOURCE_ID_KEY 40
+
+#define K8S_CONTAINER "k8s_container"
+#define K8S_NODE      "k8s_node"
+#define K8S_POD       "k8s_pod"
+
+#define STREAM_STDOUT 1
+#define STREAM_STDERR 2
+#define STREAM_UNKNOWN 3
+
+#define STDOUT "stdout"
+#define STDERR "stderr"
+
 struct flb_stackdriver {
     /* credentials */
     flb_sds_t credentials_file;
@@ -66,6 +82,17 @@ struct flb_stackdriver {
     flb_sds_t instance_id;
     flb_sds_t instance_name;
 
+    /* kubernetes specific */
+    flb_sds_t cluster_name;
+    flb_sds_t cluster_location;
+    flb_sds_t namespace_name;
+    flb_sds_t pod_name;
+    flb_sds_t container_name;
+    flb_sds_t node_name;
+    bool k8s_resource_type;
+    
+    flb_sds_t labels_key;
+
     /* other */
     flb_sds_t resource;
     flb_sds_t severity_key;
@@ -79,20 +106,28 @@ struct flb_stackdriver {
     /* upstream context for metadata end-point */
     struct flb_upstream *metadata_u;
 
+    /* plugin instance */
+    struct flb_output_instance *ins;
+
     /* Fluent Bit context */
     struct flb_config *config;
 };
 
 typedef enum {
-    EMERGENCY = 800,
-    ALERT     = 700,
-    CRITICAL  = 600,
-    ERROR     = 500,
-    WARNING   = 400,
-    NOTICE    = 300,
-    INFO      = 200,
-    DEBUG     = 100,
-    DEFAULT   = 0
+    FLB_STD_EMERGENCY = 800,
+    FLB_STD_ALERT     = 700,
+    FLB_STD_CRITICAL  = 600,
+    FLB_STD_ERROR     = 500,
+    FLB_STD_WARNING   = 400,
+    FLB_STD_NOTICE    = 300,
+    FLB_STD_INFO      = 200,
+    FLB_STD_DEBUG     = 100,
+    FLB_STD_DEFAULT   = 0
 } severity_t;
 
+struct local_resource_id_list {
+    flb_sds_t val;
+    struct mk_list _head;
+};
+
 #endif
