@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,6 +18,7 @@
  *  limitations under the License.
  */
 
+#include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_compat.h>
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_unescape.h>
@@ -42,7 +43,10 @@ static inline int key_cmp(const char *str, int len, const char *cmp) {
 static int validate_resource(const char *res)
 {
     if (strcasecmp(res, "global") != 0 &&
-        strcasecmp(res, "gce_instance") != 0) {
+        strcasecmp(res, "gce_instance") != 0 &&
+        strcasecmp(res, "k8s_container") != 0 &&
+        strcasecmp(res, "k8s_node") &&
+        strcasecmp(res, "k8s_pod")) {
         return -1;
     }
 
@@ -69,22 +73,22 @@ static int read_credentials_file(const char *creds, struct flb_stackdriver *ctx)
     ret = stat(creds, &st);
     if (ret == -1) {
         flb_errno();
-        flb_error("[out_stackdriver] cannot open credentials file: %s",
-                  creds);
+        flb_plg_error(ctx->ins, "cannot open credentials file: %s",
+                      creds);
         return -1;
     }
 
     if (!S_ISREG(st.st_mode) && !S_ISLNK(st.st_mode)) {
-        flb_error("[out_stackdriver] credentials file "
-                  "is not a valid file: %s", creds);
+        flb_plg_error(ctx->ins, "credentials file "
+                      "is not a valid file: %s", creds);
         return -1;
     }
 
     /* Read file content */
     buf = mk_file_to_buffer(creds);
     if (!buf) {
-        flb_error("[out_stackdriver] error reading credentials file: %s",
-                  creds);
+        flb_plg_error(ctx->ins, "error reading credentials file: %s",
+                      creds);
         return -1;
     }
 
@@ -99,7 +103,7 @@ static int read_credentials_file(const char *creds, struct flb_stackdriver *ctx)
 
     ret = jsmn_parse(&parser, buf, st.st_size, tokens, tok_size);
     if (ret <= 0) {
-        flb_error("[out_stackdriver] invalid JSON credentials file: %s",
+        flb_plg_error(ctx->ins, "invalid JSON credentials file: %s",
                   creds);
         flb_free(buf);
         flb_free(tokens);
@@ -108,7 +112,7 @@ static int read_credentials_file(const char *creds, struct flb_stackdriver *ctx)
 
     t = &tokens[0];
     if (t->type != JSMN_OBJECT) {
-        flb_error("[out_stackdriver] invalid JSON map on file: %s",
+        flb_plg_error(ctx->ins, "invalid JSON map on file: %s",
                   creds);
         flb_free(buf);
         flb_free(tokens);
@@ -149,7 +153,7 @@ static int read_credentials_file(const char *creds, struct flb_stackdriver *ctx)
             tmp = flb_sds_create_len(val, val_len);
             if (tmp) {
                 /* Unescape private key */
-                ctx->private_key = flb_sds_create_size(flb_sds_alloc(tmp));
+                ctx->private_key = flb_sds_create_size(val_len);
                 flb_unescape_string(tmp, flb_sds_len(tmp),
                                     &ctx->private_key);
                 flb_sds_destroy(tmp);
@@ -188,6 +192,7 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
         flb_errno();
         return NULL;
     }
+    ctx->ins = ins;
     ctx->config = config;
 
     /* Lookup credentials file */
@@ -248,28 +253,35 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
      * metadata server for default account.
      */
     if (!ctx->client_email && ctx->private_key) {
-        flb_error("[out_stackdriver] client_email is not defined");
+        flb_plg_error(ctx->ins, "client_email is not defined");
         flb_stackdriver_conf_destroy(ctx);
         return NULL;
     }
 
     if (!ctx->client_email) {
-        flb_warn("[out_stackdriver] client_email is not defined, using "
-                 "a default one");
+        flb_plg_warn(ctx->ins, "client_email is not defined, using "
+                     "a default one");
         ctx->client_email = flb_sds_create("default");
     }
     if (!ctx->private_key) {
-        flb_warn("[out_stackdriver] private_key is not defined, fetching "
-                 "it from metadata server");
+        flb_plg_warn(ctx->ins, "private_key is not defined, fetching "
+                     "it from metadata server");
         ctx->metadata_server_auth = true;
     }
 
-    /* Resource type (only 'global' and 'gce_instance' are supported) */
+    /* 
+    * Supported resource types:
+    * 'global'
+    * 'gce_instance'
+    * 'k8s_pod'
+    * 'k8s_node'
+    * 'k8s_container'
+    */
     tmp = flb_output_get_property("resource", ins);
     if (tmp) {
         if (validate_resource(tmp) != 0) {
-            flb_error("[out_stackdriver] unsupported resource type '%s'",
-                      tmp);
+            flb_plg_error(ctx->ins, "unsupported resource type '%s'",
+                          tmp);
             flb_stackdriver_conf_destroy(ctx);
             return NULL;
         }
@@ -284,6 +296,41 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
         ctx->severity_key = flb_sds_create(tmp);
     }
 
+    if (flb_sds_cmp(ctx->resource, "k8s_container", 
+                    flb_sds_len(ctx->resource)) == 0 || 
+        flb_sds_cmp(ctx->resource, "k8s_node", 
+                    flb_sds_len(ctx->resource)) == 0 ||
+        flb_sds_cmp(ctx->resource, "k8s_pod", 
+                    flb_sds_len(ctx->resource)) == 0) {
+        
+        ctx->k8s_resource_type = FLB_TRUE;
+
+        tmp = flb_output_get_property("k8s_cluster_name", ins);
+        if (tmp) {
+            ctx->cluster_name = flb_sds_create(tmp);
+        }
+
+        tmp = flb_output_get_property("k8s_cluster_location", ins);
+        if (tmp) {
+            ctx->cluster_location = flb_sds_create(tmp);
+        }
+
+        if (!ctx->cluster_name || !ctx->cluster_location) {
+            flb_plg_error(ctx->ins, "missing k8s_cluster_name "
+                          "or k8s_cluster_location in configuration");
+            flb_stackdriver_conf_destroy(ctx);
+            return NULL;
+        }
+    }
+    
+    tmp = flb_output_get_property("labels_key", ins);
+    if (tmp) {
+        ctx->labels_key = flb_sds_create(tmp);
+    }
+    else {
+        ctx->labels_key = flb_sds_create(DEFAULT_LABELS_KEY);
+    }
+
     return ctx;
 }
 
@@ -293,6 +340,15 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
         return -1;
     }
 
+    if (ctx->k8s_resource_type){
+        flb_sds_destroy(ctx->namespace_name);
+        flb_sds_destroy(ctx->pod_name);
+        flb_sds_destroy(ctx->container_name);
+        flb_sds_destroy(ctx->node_name);
+        flb_sds_destroy(ctx->cluster_name);
+        flb_sds_destroy(ctx->cluster_location);
+    }
+
     flb_sds_destroy(ctx->credentials_file);
     flb_sds_destroy(ctx->type);
     flb_sds_destroy(ctx->project_id);
@@ -304,15 +360,24 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
     flb_sds_destroy(ctx->token_uri);
     flb_sds_destroy(ctx->resource);
     flb_sds_destroy(ctx->severity_key);
+    flb_sds_destroy(ctx->labels_key);
 
     if (ctx->o) {
         flb_oauth2_destroy(ctx->o);
     }
+
     if (ctx->metadata_server_auth) {
-      flb_sds_destroy(ctx->zone);
-      flb_sds_destroy(ctx->instance_id);
+        flb_sds_destroy(ctx->zone);
+        flb_sds_destroy(ctx->instance_id);
     }
 
+    if (ctx->metadata_u) {
+        flb_upstream_destroy(ctx->metadata_u);
+    }
+
+    if (ctx->u) {
+        flb_upstream_destroy(ctx->u);
+    }
     flb_free(ctx);
 
     return 0;
