@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,13 +21,14 @@
 #ifndef FLB_OUT_STDOUT
 #define FLB_OUT_STDOUT
 
-#include <fluent-bit/flb_info.h>
+#include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_sds.h>
 
-struct flb_out_stdout_config {
+struct flb_stdout {
     int out_format;
     int json_date_format;
     flb_sds_t json_date_key;
+    struct flb_output_instance *ins;
 };
 
 #endif
