@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -88,6 +88,15 @@ int flb_engine_flush(struct flb_config *config,
     return 0;
 }
 
+static void cb_engine_sched_timer(struct flb_config *ctx, void *data)
+{
+    (void) data;
+
+    /* Upstream connections timeouts handling */
+    flb_upstream_conn_timeouts(ctx);
+}
+
+
 static inline int flb_engine_manager(flb_pipefd_t fd, struct flb_config *config)
 {
     int ret;
@@ -288,7 +297,7 @@ static FLB_INLINE int flb_engine_handle_event(flb_pipefd_t fd, int mask,
         }
         else if (config->ch_manager[0] == fd) {
             ret = flb_engine_manager(fd, config);
-            if (ret == FLB_ENGINE_STOP) {
+            if (ret == FLB_ENGINE_STOP || ret == FLB_ENGINE_EV_STOP) {
                 return FLB_ENGINE_STOP;
             }
         }
@@ -307,6 +316,7 @@ static FLB_INLINE int flb_engine_handle_event(flb_pipefd_t fd, int mask,
         }
 #endif
 
+        /* Stream processor event ? */
 #ifdef FLB_HAVE_STREAM_PROCESSOR
         if (config->stream_processor_ctx) {
             ret = flb_sp_fd_event(fd, config->stream_processor_ctx);
@@ -401,21 +411,6 @@ int flb_engine_start(struct flb_config *config)
         return -1;
     }
 
-    /* Start the Storage engine */
-    ret = flb_storage_create(config);
-    if (ret == -1) {
-        return -1;
-    }
-
-    flb_info("[engine] started (pid=%i)", getpid());
-
-    /* Debug coroutine stack size */
-    flb_utils_bytes_to_human_readable_size(config->coro_stack_size,
-                                           (char *) &tmp, sizeof(tmp));
-    flb_debug("[engine] coroutine stack size: %lu bytes (%s)",
-              config->coro_stack_size, tmp);
-    flb_thread_prepare();
-
     /* Create the event loop and set it in the global configuration */
     evl = mk_event_loop_create(256);
     if (!evl) {
@@ -423,6 +418,15 @@ int flb_engine_start(struct flb_config *config)
     }
     config->evl = evl;
 
+
+    flb_info("[engine] started (pid=%i)", getpid());
+
+    /* Debug coroutine stack size */
+    flb_utils_bytes_to_human_readable_size(config->coro_stack_size,
+                                           tmp, sizeof(tmp));
+    flb_debug("[engine] coroutine stack size: %u bytes (%s)",
+              config->coro_stack_size, tmp);
+
     /*
      * Create a communication channel: this routine creates a channel to
      * signal the Engine event loop. It's useful to stop the event loop
@@ -437,24 +441,36 @@ int flb_engine_start(struct flb_config *config)
         return -1;
     }
 
+    /* Start the Storage engine */
+    ret = flb_storage_create(config);
+    if (ret == -1) {
+        return -1;
+    }
+
     /* Initialize input plugins */
-    flb_input_initialize_all(config);
+    ret = flb_input_init_all(config);
+    if (ret == -1) {
+        return -1;
+    }
+
+    /* Initialize filter plugins */
+    ret = flb_filter_init_all(config);
+    if (ret == -1) {
+        return -1;
+    }
 
     /* Inputs pre-run */
     flb_input_pre_run_all(config);
 
     /* Initialize output plugins */
-    ret = flb_output_init(config);
-    if (ret == -1 && config->support_mode == FLB_FALSE) {
+    ret = flb_output_init_all(config);
+    if (ret == -1) {
         return -1;
     }
 
     /* Outputs pre-run */
     flb_output_pre_run(config);
 
-    /* Initialize filter plugins */
-    flb_filter_initialize_all(config);
-
     /* Create and register the timer fd for flush procedure */
     event = &config->event_flush;
     event->mask = MK_EVENT_EMPTY;
@@ -476,6 +492,12 @@ int flb_engine_start(struct flb_config *config)
         return -1;
     }
 
+#ifdef FLB_HAVE_METRICS
+    if (config->storage_metrics == FLB_TRUE) {
+        config->storage_metrics_ctx = flb_storage_metrics_create(config);
+    }
+#endif
+
     /* Initialize collectors */
     flb_input_collectors_start(config);
 
@@ -514,6 +536,18 @@ int flb_engine_start(struct flb_config *config)
     }
 #endif
 
+    /*
+     * Sched a permanent callback triggered every 1.5 second to let other
+     * Fluent Bit components run tasks at that interval.
+     */
+    ret = flb_sched_timer_cb_create(config,
+                                    FLB_SCHED_TIMER_CB_PERM,
+                                    1500, cb_engine_sched_timer, config);
+    if (ret == -1) {
+        flb_error("[engine] could not schedule permanent callback");
+        return -1;
+    }
+
     /* Signal that we have started */
     flb_engine_started(config);
 
@@ -525,13 +559,15 @@ int flb_engine_start(struct flb_config *config)
                 if (ret == FLB_ENGINE_STOP) {
                     /*
                      * We are preparing to shutdown, we give a graceful time
-                     * of (default 5) seconds to process any pending event.
+                     * of 'config->grace' seconds to process any pending event.
                      */
                     event = &config->event_shutdown;
                     event->mask = MK_EVENT_EMPTY;
                     event->status = MK_EVENT_NONE;
-                    config->shutdown_fd = mk_event_timeout_create(evl, config->grace, 0, event);
-
+                    config->shutdown_fd = mk_event_timeout_create(evl,
+                                                                  config->grace,
+                                                                  0,
+                                                                  event);
                     flb_warn("[engine] service will stop in %u seconds", config->grace);
                 }
                 else if (ret == FLB_ENGINE_SHUTDOWN) {
@@ -540,7 +576,27 @@ int flb_engine_start(struct flb_config *config)
                         mk_event_timeout_destroy(config->evl,
                                                  &config->event_shutdown);
                     }
-                    return flb_engine_shutdown(config);
+
+                    /*
+                     * Grace period has finished, but we need to check if there is
+                     * any pending running task. A running task is associated to an
+                     * output co-routine, since we don't know what's the state or
+                     * resources allocated by that co-routine, the best thing is to
+                     * wait again for the grace period and re-check again.
+                     */
+                    ret = flb_task_running_count(config);
+                    if (ret > 0) {
+                        flb_warn("[engine] shutdown delayed, grace period has "
+                                 "finished but some tasks are still running.");
+                        flb_task_running_print(config);
+                        flb_engine_exit(config);
+                    }
+                    else {
+                        ret = config->exit_status_code;
+                        flb_engine_shutdown(config);
+                        config = NULL;
+                        return ret;
+                    }
                 }
             }
             else if (event->type & FLB_ENGINE_EV_SCHED) {
@@ -598,7 +654,6 @@ int flb_engine_shutdown(struct flb_config *config)
     flb_input_exit_all(config);
     flb_output_exit(config);
 
-
     /* Destroy the storage context */
     flb_storage_destroy(config);
 
@@ -631,3 +686,9 @@ int flb_engine_exit(struct flb_config *config)
     ret = flb_pipe_w(config->ch_manager[1], &val, sizeof(uint64_t));
     return ret;
 }
+
+int flb_engine_exit_status(struct flb_config *config, int status)
+{
+    config->exit_status_code = status;
+    return flb_engine_exit(config);
+}
