@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -352,7 +352,7 @@ char *flb_oauth2_token_get(struct flb_oauth2 *ctx)
     /* Issue request */
     ret = flb_http_do(c, &b_sent);
     if (ret != 0) {
-        flb_warn("[oauth2] cannot issue request, http_do=%i, ret");
+        flb_warn("[oauth2] cannot issue request, http_do=%i", ret);
     }
     else {
         flb_info("[oauth2] HTTP Status=%i", c->resp.status);
