@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -88,7 +88,7 @@ static int logfmt_parser(struct flb_parser *parser,
 
     while (c < end) {
         /* garbage */
-        while (!ident_byte[*c] && (c < end)) {
+        while ((c < end) && !ident_byte[*c]) {
             c++;
         }
         if (c == end) {
@@ -96,14 +96,19 @@ static int logfmt_parser(struct flb_parser *parser,
         }
         /* key */
         key = c;
-        while (ident_byte[*c] && (c < end)) {
+        while ((c < end) && ident_byte[*c]) {
             c++;
         }
+        if (c == end) {
+            break;
+        }
+
         key_len = c - key;
         /* value */
         value_len = 0;
         value_str = FLB_FALSE;
         value_escape =  FLB_FALSE;
+
         if (*c == '=') {
             c++;
             if (c < end) {
@@ -134,7 +139,7 @@ static int logfmt_parser(struct flb_parser *parser,
                 }
                 else {
                    value = c;
-                   while (ident_byte[*c] && (c < end)) {
+                   while ((c < end) && ident_byte[*c]) {
                       c++;
                    }
                    value_len = c - value;
@@ -152,9 +157,9 @@ static int logfmt_parser(struct flb_parser *parser,
                     ret = flb_parser_time_lookup((const char *) value, value_len,
                                                   0, parser, &tm, tmfrac);
                     if (ret == -1) {
-                       flb_error("[parser:%s] Invalid time format %s.",
-                                 parser->name, parser->time_fmt);
-                       return -1;
+                        flb_error("[parser:%s] Invalid time format %s",
+                                  parser->name, parser->time_fmt_full);
+                        return -1;
                     }
                     *time_lookup = flb_parser_tm2time(&tm);
                 }
