@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -81,8 +81,8 @@ static void cb_results(const char *name, const char *value,
                 }
                 memcpy(tmp, value, vlen);
                 tmp[vlen] = '\0';
-                flb_warn("[parser:%s] Invalid time format %s for '%s'.",
-                         parser->name, parser->time_fmt, tmp);
+                flb_warn("[parser:%s] invalid time format %s for '%s'",
+                         parser->name, parser->time_fmt_full, tmp);
                 pcb->num_skipped++;
                 return;
             }
