@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -49,7 +49,7 @@ void flb_utils_error(int err)
         msg = "Configuration file contains format errors";
         break;
     case FLB_ERR_CFG_FILE_STOP:
-        msg = "Configuration file contain errors";
+        msg = "Configuration file contains errors";
         break;
     case FLB_ERR_CFG_FLUSH:
         msg = "Invalid flush value";
@@ -154,7 +154,7 @@ int flb_utils_set_daemon(struct flb_config *config)
 	}
 
     /* Our last STDOUT messages */
-    flb_info("switching to background mode (PID=%lu)", getpid());
+    flb_info("switching to background mode (PID=%ld)", (long) getpid());
 
     fclose(stderr);
     fclose(stdout);
@@ -307,6 +307,13 @@ struct mk_list *flb_utils_split(const char *line, int separator, int max_split)
     return list;
 }
 
+void flb_utils_split_free_entry(struct flb_split_entry *entry)
+{
+    mk_list_del(&entry->_head);
+    flb_free(entry->value);
+    flb_free(entry);
+}
+
 void flb_utils_split_free(struct mk_list *list)
 {
     struct mk_list *tmp;
@@ -315,9 +322,7 @@ void flb_utils_split_free(struct mk_list *list)
 
     mk_list_foreach_safe(head, tmp, list) {
         entry = mk_list_entry(head, struct flb_split_entry, _head);
-        mk_list_del(&entry->_head);
-        flb_free(entry->value);
-        flb_free(entry);
+        flb_utils_split_free_entry(entry);
     }
 
     flb_free(list);
@@ -375,6 +380,10 @@ int64_t flb_utils_size_to_bytes(const char *size)
         return -1;
     }
 
+    if (strcasecmp(size, "false") == 0) {
+        return 0;
+    }
+
     len = strlen(size);
     val = atoll(size);
 
@@ -454,8 +463,13 @@ int flb_utils_bool(const char *val)
         strcasecmp(val, "yes") == 0) {
         return FLB_TRUE;
     }
+    else if (strcasecmp(val, "false") == 0 ||
+             strcasecmp(val, "off") == 0 ||
+             strcasecmp(val, "no") == 0) {
+        return FLB_FALSE;
+    }
 
-    return FLB_FALSE;
+    return -1;
 }
 
 /* Convert a 'string' time seconds.nanoseconds to int and long values */
@@ -716,6 +730,18 @@ int flb_utils_write_str_buf(const char *str, size_t str_len, char **out, size_t
     return 0;
 }
 
+static char *flb_copy_host(const char *string, int pos_init, int pos_end)
+{
+    if (string[pos_init] == '[') {            /* IPv6 */
+        if (string[pos_end-1] != ']')
+            return NULL;
+
+        return mk_string_copy_substr(string, pos_init + 1, pos_end - 1);
+    }
+    else
+        return mk_string_copy_substr(string, pos_init, pos_end);
+}
+
 int flb_utils_url_split(const char *in_url, char **out_protocol,
                         char **out_host, char **out_port, char **out_uri)
 {
@@ -757,7 +783,7 @@ int flb_utils_url_split(const char *in_url, char **out_protocol,
     }
 
     if (tmp) {
-        host = mk_string_copy_substr(p, 0, tmp - p);
+        host = flb_copy_host(p, 0, tmp - p);
         if (!host) {
             flb_errno();
             goto error;
@@ -778,15 +804,24 @@ int flb_utils_url_split(const char *in_url, char **out_protocol,
     else {
         tmp = strchr(p, '/');
         if (tmp) {
-            host = mk_string_copy_substr(p, 0, tmp - p);
+            host = flb_copy_host(p, 0, tmp - p);
             uri = flb_strdup(tmp);
         }
         else {
-            host = flb_strdup(p);
+            host = flb_copy_host(p, 0, strlen(p));
             uri = flb_strdup("/");
         }
     }
 
+    if (!port) {
+        if (strcmp(protocol, "http") == 0) {
+            port = flb_strdup("80");
+        }
+        else if (strcmp(protocol, "https") == 0) {
+            port = flb_strdup("443");
+        }
+    }
+
     *out_protocol = protocol;
     *out_host = host;
     *out_port = port;
